/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.application.configuration;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfiguration;
import com.frojasg1.general.exceptions.ConfigurationException;
import java.io.File;
import java.util.Properties;

/**
 *
 * @author Usuario
 */
public class ApplicationConfiguration	extends BaseApplicationConfiguration
{
//	protected static final String RESOURCE_FOR_APPLICATION_ICON="com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png";

	public static final String CONF_LAST_DIRECTORY = "LAST_DIRECTORY";

//	protected static final String APPLICATION_GROUP = "general";
	protected static final String GLOBAL_CONF_FILE_NAME = "GlobalConfiguration.properties";

	public static final String sa_MAIN_FOLDER = ".frojasg1.apps";
	public static String sa_APPLICATION_NAME = "Textcrypt";
	public static final String sa_CONFIGURATION_GROUP = "Configuration";
	public static final String sa_PROPERTIES_PATH_IN_JAR="com/frojasg1/applications/textcrypt/lan";

	public static final String VERSION = "v1.3";

	public static final String ES_LANGUAGE = "ES";	 // Spanish language
	public static final String EN_LANGUAGE = "EN";	 // English language

	public static final String CONF_LANGUAGE = "LANGUAGE";
	public static final String CONF_ADDITIONAL_LANGUAGE = "ADDITIONAL_LANGUAGE";

	public static final String CONF_LICENSES_HAVE_BEEN_ACCEPTED = "LICENSES_HAVE_BEEN_ACCEPTED";

	private static ApplicationConfiguration a_instance = null;

	private ApplicationConfiguration()
	{
//		super( sa_MAIN_FOLDER, sa_APPLICATION_NAME, APPLICATION_GROUP, null, GLOBAL_CONF_FILE_NAME );
		super( sa_MAIN_FOLDER, sa_APPLICATION_NAME + "/" + VERSION, sa_CONFIGURATION_GROUP, null, GLOBAL_CONF_FILE_NAME );
	}
	
	public static ApplicationConfiguration create( String applicationName )
	{
		sa_APPLICATION_NAME = applicationName;
		a_instance = new ApplicationConfiguration();

		return( a_instance );
	}

	public static ApplicationConfiguration instance()
	{
/*		if( a_instance == null )
		{
			a_instance = new ApplicationConfiguration();
			a_instance.loadDefaultProperties();
		}
*/		
		return( a_instance );
	}

	public Properties M_getDefaultProperties( String language )
	{
		Properties result = super.M_getDefaultProperties( language );

		result.setProperty(CONF_LANGUAGE, EN_LANGUAGE );
		result.setProperty(CONF_ADDITIONAL_LANGUAGE, "CAT" );
		result.setProperty(CONF_LICENSES_HAVE_BEEN_ACCEPTED, "0" );

		return( result );
	}

	@Override
	public String getConfigurationMainFolder()
	{
		return( sa_MAIN_FOLDER );
	}

	@Override
	public String getApplicationName()
	{
		return( sa_APPLICATION_NAME );
	}

	@Override
	public String getApplicationGroup()
	{
		return( sa_CONFIGURATION_GROUP );
	}

	@Override
	public String getResourceNameForApplicationIcon()
	{
//		return( RESOURCE_FOR_APPLICATION_ICON );
		return( null );
	}

	@Override
	public String getInternationalPropertiesPathInJar()
	{
		return( sa_PROPERTIES_PATH_IN_JAR );
//		return( null );
	}
	
	public String getAdditionalLanguage()
	{
		return( M_getStrParamConfiguration( CONF_ADDITIONAL_LANGUAGE ) );
	}
	
	public void setAdditionalLanguage( String value )
	{
		M_setStrParamConfiguration( CONF_ADDITIONAL_LANGUAGE, value );
	}

	@Override
	protected void changeLanguage_internal_simple( String language ) throws ConfigurationException
	{
		
	}

	@Override
	public String getApplicationNameFolder()
	{
		return( getApplicationName() + "/" + getApplicationVersion() );
	}

	@Override
	public String getApplicationVersion()
	{
		return( VERSION );
	}

	@Override
	public boolean getLicensesHaveBeenAccepted()
	{
		return( this.M_getBoolParamConfiguration(CONF_LICENSES_HAVE_BEEN_ACCEPTED) );
	}

	@Override
	public void setLicencesHaveBeenAccepted(boolean value)
	{
		this.M_setBoolParamConfiguration( CONF_LICENSES_HAVE_BEEN_ACCEPTED, value );
	}

	@Override
	public String getDefaultLanguageConfigurationFolder(String language)
	{
		String result = System.getProperty("user.home") + File.separator  +
						getConfigurationMainFolder() + File.separator +
						getApplicationNameFolder() + File.separator +
						getApplicationGroup() + File.separator +
						language;

		return( result );
	}

	@Override
	public String getDefaultLanguage()
	{
		return( EN_LANGUAGE );
	}

}
