/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;

/**
 *
 * @author Usuario
 */
public class GenericFileFunctions
{
	public static String  DIR_SEPARATOR = System.getProperty( "file.separator" );

	static public String getExtension( File file )
    {
        String fileName = file.toString();
        return( getExtension( fileName ) );
    }

    static public String getExtension( String fileName )
    {
        String extension = "";
        
        String longFileName = fileName;
        String shortFileName = null;
        int pos = longFileName.lastIndexOf( File.separator );
        while( (pos>0) && (pos==(longFileName.length()-1) ))
        {
            longFileName = longFileName.substring( 0, longFileName.length()-1 );
            pos = longFileName.lastIndexOf( File.separator );
        }

        if( (pos >= 0) && (pos<(longFileName.length()-1) ) )
        {
            shortFileName = longFileName.substring( pos + 1 );
        }
        else if( pos < 0 )
        {
            shortFileName = longFileName;
        }

        pos = shortFileName.lastIndexOf( "." );
        if( (pos>0) && (pos<(shortFileName.length()-1) ))
        {
            extension = shortFileName.substring( pos+1, shortFileName.length() );
        }

/*
        System.out.println( "File:" + fileName +
                            "shortFileName: " + shortFileName +
                            ", extension: " + extension +
                            ", separator: " + File.separator );
*/
        return( extension );
    }
	
    static public String cutOffExtension( String fileName )
    {
        String extension = getExtension( fileName );
        int lenExt = (extension.length() > 0 ? extension.length() + 1 : 0 );    // para el punto
        String outputFileName = fileName.substring(0, fileName.length()-lenExt );
        System.out.println( "fileName: " + fileName + "without extension: " + outputFileName );
        
        return( outputFileName );
    }

	public static String convertFolderSeparator( String compoundFolder )
	{
		String result = StringFunctions.replaceSetOfChars(compoundFolder, "\\/", DIR_SEPARATOR + DIR_SEPARATOR );
		return( result );
	}

	public static boolean isDirectory( String dirname )
	{
		File dir = new File( dirname );
		return( dir.exists() && dir.isDirectory() );
	}

	public static boolean isFile( String fileName )
	{
		File file = new File( fileName );
		return( file.exists() && file.isFile() );
	}

	public static void copyFile( String sourceFileName, String destFileName ) throws IOException
	{
		boolean overwrite = true;
		copyFile( sourceFileName, destFileName, overwrite );
	}

	// http://stackoverflow.com/questions/106770/standard-concise-way-to-copy-a-file-in-java
	public static void copyFile( String sourceFileName, String destFileName, boolean overwrite ) throws IOException
	{
		File sourceFile = new File( sourceFileName );
		File destFile = new File( destFileName );
		
		if(!destFile.exists())
		{
			destFile.createNewFile();
		}
		else if( !overwrite )
		{
			return;
		}

		FileChannel source = null;
		FileChannel destination = null;

		try {
			source = new FileInputStream(sourceFile).getChannel();
			destination = new FileOutputStream(destFile).getChannel();
			destination.transferFrom(source, 0, source.size());
		}
		finally {
			if(source != null) {
				source.close();
			}
			if(destination != null) {
				destination.close();
			}
		}
	}

	public static void copyDirectoryRecursive( String oldDir, String newDir,
												boolean onlyForNonExistingNewDir,
												boolean overwrite ) throws IOException
	{
		File oDir = new File( oldDir );
		if( !oDir.exists() || !oDir.isDirectory() )
			throw( new RuntimeException( "Error: " + oldDir + " does not exist or it is not a directory." ) );

		File nDir = new File( newDir );
		if( nDir.exists() && onlyForNonExistingNewDir )
			throw( new RuntimeException( "Error: new directory existed previously." ) );

		nDir.mkdirs();
		File files[] = oDir.listFiles();
		for( int ii=0; ii<files.length; ii++ )
		{
			if( files[ii].isFile() )
				copyFile( files[ii].getPath(), newDir + DIR_SEPARATOR + files[ii].getName(), overwrite );
			else if( files[ii].isDirectory() )
				copyDirectoryRecursive( files[ii].getPath(), newDir + DIR_SEPARATOR + files[ii].getName(),
										onlyForNonExistingNewDir, overwrite );
		}
	}

	public static void copyDirectoryRecursive( String oldDir, String newDir ) throws IOException
	{
		boolean onlyForNonExistingNewDir = true;
		boolean overwrite = false;
		copyDirectoryRecursive( oldDir, newDir,	onlyForNonExistingNewDir, overwrite );
	}

	public static void eraseDirCompletely( String dir ) throws IOException
	{
		File oDir = new File( dir );
		if( !oDir.exists() || !oDir.isDirectory() )
			throw( new RuntimeException( "Error: " + dir + " does not exist or it is not a directory." ) );

		File files[] = oDir.listFiles();
		for( int ii=0; ii<files.length; ii++ )
		{
			if( files[ii].isFile() )
			{
				  files[ii].delete();
			}
			else if( files[ii].isDirectory() )
			{
				eraseDirCompletely( files[ii].getPath() );
			}
		}
		
		oDir.delete();
	}
	
	public static void eraseRecursive( String dir, String fileToErase ) throws IOException
	{
		File oDir = new File( dir );
		if( !oDir.exists() || !oDir.isDirectory() )
			throw( new RuntimeException( "Error: " + dir + " does not exist or it is not a directory." ) );

		File files[] = oDir.listFiles();
		for( int ii=0; ii<files.length; ii++ )
		{
			if( files[ii].isFile() && files[ii].getName().equals( fileToErase ) )
			{
				  files[ii].delete();
			}
			else if( files[ii].isDirectory() )
			{
				eraseRecursive( files[ii].getPath(), fileToErase );
			}
		}
	}

	public static String getTextContentsOfAFile( String fileName, String charsetName ) throws IOException
	{
		String result = null;
		
		File file = new File( fileName );
		if( file.exists() && file.isFile() )		// read the file as binary
		{
			byte[] fileData = new byte[(int) file.length()];
			
			DataInputStream dis = null;
			FileInputStream fis = null;
			try
			{
				fis = new FileInputStream( file );
				dis = new DataInputStream( fis );
				dis.readFully(fileData);
				dis.close();
			}
			catch( IOException ioe )
			{
				throw( ioe );
			}
			finally
			{
				if( dis != null )
					dis.close();
				else if( fis != null )
					fis.close();
			}

			result = new String( fileData, charsetName );	// convert binary data into String with a particular Charset
		}

		return( result );
	}

}
