/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.generic.languages;

import com.frojasg1.general.ArrayFunctions;
import com.frojasg1.general.CollectionFunctions;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public class ObtainAvailableLanguages_base implements ObtainAvailableLanguages_int
{
	protected static ObtainAvailableLanguages_base _instance = null;

	protected String _rootLanguagePackage = null;
	protected String _rootLanguageConfigurationPathInDisk = null;

	protected String[] _basicLanguages = null;
	protected ArrayList<String> _basicLanguagesList = new ArrayList<String>( );
	protected String[] _availableLanguagesInJar = null;
	protected ArrayList<String> _availableLanguagesInJarList = new ArrayList<String>( );

	protected List<String> _availableLanguagesInDiskList = null;

	public static ObtainAvailableLanguages_base create( String[] basicLanguages,
													String[] availableLanguagesInJar,
													String rootLanguageConfigurationPathInDisk )
	{
		_instance = new ObtainAvailableLanguages_base();

		_instance.setRootLanguageConfigurationPathInDisk( rootLanguageConfigurationPathInDisk );

		if( basicLanguages != null )
			_instance.setBasicLanguages( basicLanguages );

		if( availableLanguagesInJar != null )
			_instance.setAvailableLanguagesInJar( availableLanguagesInJar );

		return( _instance );
	}

	public static ObtainAvailableLanguages_base instance()
	{
		return( _instance );
	}

	public void setBasicLanguages( String[] basicLanguages )
	{
		if( basicLanguages != null )
		{
			_basicLanguages = new String[basicLanguages.length];
			System.arraycopy( basicLanguages, 0, _basicLanguages, 0, basicLanguages.length );
			_basicLanguagesList = new ArrayList<String>( Arrays.asList( _basicLanguages ) );
		}
	}

	public void setAvailableLanguagesInJar( String[] availableLanguagesInJar )
	{
		if( availableLanguagesInJar != null )
		{
			_availableLanguagesInJar = new String[availableLanguagesInJar.length];
			System.arraycopy( availableLanguagesInJar, 0, _availableLanguagesInJar, 0, availableLanguagesInJar.length );
			_availableLanguagesInJarList = new ArrayList<String>( Arrays.asList( _availableLanguagesInJar ) );
		}
	}

	public void setRootLanguageConfigurationPathInDisk( String rootLanguageConfigurationPathInDisk )
	{
		_rootLanguageConfigurationPathInDisk = rootLanguageConfigurationPathInDisk;
	}

	public String getRootLanguageConfigurationPathInDisk()
	{
		return( _rootLanguageConfigurationPathInDisk );
	}

	public ListOfLanguagesResult_int getTotalListOfAvailableLanguages()
	{
		return( getTotalListOfAvailableLanguages( _rootLanguagePackage, _rootLanguageConfigurationPathInDisk ) );
	}

	public ListOfLanguagesResult_int getTotalListOfAvailableLanguages(String rootLanguagePackage,
													String rootLanguageConfigurationPathInDisk )
	{
		List<String> list1 = getListOfAvailableLanguagesInPackage();
		List<String> list2 = getListOfAvailableLanguagesConfiguredInDisk( rootLanguageConfigurationPathInDisk );

		list1.addAll( list2 );

		CollectionFunctions.sortNoCaseSensitive(list1);
		CollectionFunctions.removeRepeatedSortedPreviously(list1);

		List<String> removedItems = CollectionFunctions.removeItemsThatExistInTheSecondList( list1, _basicLanguagesList );
		List<String> fixedLanguageList = CollectionFunctions.leaveOnlyExistingInTheSecondList( _basicLanguagesList, removedItems );
		List<String> otherLanguagesList = list1;

		ListOfLanguagesResult result = new ListOfLanguagesResult( fixedLanguageList, otherLanguagesList );

		return( result );
	}

	public List<String> getListOfAvailableLanguagesInPackage()
	{
		return( new ArrayList<String>( _availableLanguagesInJarList ) );
	}

	public List<String> getListOfAvailableLanguagesConfiguredInDisk()
	{
		if( _availableLanguagesInDiskList == null )
			_availableLanguagesInDiskList = getListOfAvailableLanguagesConfiguredInDisk( _rootLanguageConfigurationPathInDisk );

		return( _availableLanguagesInDiskList );
	}

	public List<String> getListOfAvailableLanguagesConfiguredInDisk( String rootLanguageConfigurationPathInDisk )
	{
		ArrayList<String> result = new ArrayList<String>();

		File folder = new File( rootLanguageConfigurationPathInDisk );
		if( folder.isDirectory() )
		{
			File[] contents = folder.listFiles();

			for( int ii=0; ii<contents.length; ii++ )
			{
				if( contents[ii].isDirectory() )
				{
					result.add( contents[ii].getName() );
				}
			}

			CollectionFunctions.sortNoCaseSensitive(result);
		}

		return( result );
	}

	public String[] getTotalArrayOfAvailableLanguages()
	{
		ListOfLanguagesResult_int lolr = getTotalListOfAvailableLanguages();
		return( ArrayFunctions.getArrayJoiningLists( lolr.getListOfFixedLanguages(), lolr.getListOfOtherLanguages() ) );
	}

	public void newLanguageSetToConfiguration( String language )
	{
		getListOfAvailableLanguagesConfiguredInDisk();
		
		if( ! CollectionFunctions.itemExistsInList( _availableLanguagesInDiskList, language ) )
			_availableLanguagesInDiskList.add( language );
	}

	public static class ListOfLanguagesResult implements ListOfLanguagesResult_int
	{
		protected List<String> _listOfFixedLanguages = null;
		protected List<String> _listOfOtherLanguages = null;

		public ListOfLanguagesResult( List<String> listOfFixedLanguages, List<String> listOfOtherLanguages )
		{
			_listOfFixedLanguages = listOfFixedLanguages;
			_listOfOtherLanguages = listOfOtherLanguages;
		}

		public List<String> getListOfFixedLanguages()
		{
			return( _listOfFixedLanguages );
		}

		public List<String> getListOfOtherLanguages()
		{
			return( _listOfOtherLanguages );
		}
	}

}
