/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.license;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.ArrayFunctions;
import com.frojasg1.general.ResourceFunctions;
import com.frojasg1.general.desktop.files.FileFunctions;
import com.frojasg1.general.exceptions.ConfigurationException;
import com.frojasg1.generic.GenericFunctions;
import com.hotmail.frojasg1.applications.common.components.internationalization.InternException;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJDialog;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import java.io.File;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.text.Document;

/**
 *
 * @author Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 */
public abstract class GenericLicenseJDialog extends InternationalizedJDialog
{

	protected boolean a_initializing = true;

	protected boolean _acceptedLicensesAndStart = false;

	protected SimpleRtfDocJPanel _rtfDocJPanel = null;

	protected boolean _showToAcceptTheLicense = false;
	
	protected String[] _arrayOfLanguages = null;
	
	/**
	 * Creates new form LicenseJFrame
	 */

	public GenericLicenseJDialog( JFrame parent, BaseApplicationConfigurationInterface appConf,
									boolean showToAcceptTheLicense )
	{
		super( parent, true, appConf );

		_showToAcceptTheLicense = showToAcceptTheLicense;

		initComponents();

		initOwnComponents();

		a_initializing = true;
		M_fillLanguageComboBox();
		a_initializing = false;

		setWindowConfiguration();
	}

	public GenericLicenseJDialog( JDialog parent, BaseApplicationConfigurationInterface appConf,
									boolean showToAcceptTheLicense )
	{
		super( parent, true, appConf );

		_showToAcceptTheLicense = showToAcceptTheLicense;

		initComponents();

		initOwnComponents();

		a_initializing = true;
		M_fillLanguageComboBox();
		a_initializing = false;

		setWindowConfiguration();
	}

	@Override
	public void setVisible( boolean value )
	{
		if( value )
			_acceptedLicensesAndStart = false;

		super.setVisible( value );
	}

	protected void initOwnComponents()
	{
		if( ! _showToAcceptTheLicense )
		{
			int extraHeight = jPanel2.getHeight();
			jPanel2.setVisible( false );
			jPanel1.setSize( jPanel1.getWidth(), jPanel1.getHeight() + extraHeight );
		}

		_rtfDocJPanel = new SimpleRtfDocJPanel();
		jPanel1.add( _rtfDocJPanel );

		_rtfDocJPanel.setBounds( 0, 0, jPanel1.getWidth(), jPanel1.getHeight() );
	}

	public boolean getLicensesHaveBeenAcceptedAndWeHaveToStart()
	{
		return( _acceptedLicensesAndStart );
	}

	protected void setWindowConfiguration( )
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();
		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( _rtfDocJPanel, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
//			mapRRCI.putAll( _rtfDocJPanel.getMapResizeRelocateComponentItem() );

			mapRRCI.putResizeRelocateComponentItem( jPanel2, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.MOVE_TO_BOTTOM );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		createInternationalization(getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									getBaseConfFileName(),
									this,
									getParent(),
									null,
									true,
									mapRRCI );
		
		a_intern.setMaxWindowWidthNoLimit( false );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
	}

	protected void M_fillLanguageComboBox()
	{
		_arrayOfLanguages = GenericFunctions.instance().getObtainAvailableLanguages().getTotalArrayOfAvailableLanguages();

		if( (!a_initializing) && ( jCb_language.getItemCount() == _arrayOfLanguages.length ) )
		{}
		else
		{
			int selectedIndex = -1;

			String language = null;
			if( a_initializing )
				language = getAppliConf().getLanguage();
			else
				language = (String) jCb_language.getSelectedItem();

			selectedIndex = ArrayFunctions.getFirstIndexOfEquals(_arrayOfLanguages, language );

			jCb_language.setModel(new javax.swing.DefaultComboBoxModel(_arrayOfLanguages));

			if( selectedIndex >= 0 ) jCb_language.setSelectedIndex(selectedIndex);
		}
	}
	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jL_language = new javax.swing.JLabel();
        jCb_language = new javax.swing.JComboBox();
        jCB_accept = new javax.swing.JCheckBox();
        jB_startApp = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setMaximumSize(new java.awt.Dimension(700, 5000));
        setMinimumSize(new java.awt.Dimension(460, 511));
        setName(""); // NOI18N
        setPreferredSize(new java.awt.Dimension(517, 670));
        getContentPane().setLayout(null);

        jPanel1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jPanel1.setLayout(null);
        getContentPane().add(jPanel1);
        jPanel1.setBounds(10, 10, 480, 530);

        jPanel2.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jPanel2.setLayout(null);

        jL_language.setText("Language :");
        jL_language.setName("jL_language"); // NOI18N
        jPanel2.add(jL_language);
        jL_language.setBounds(10, 16, 80, 14);

        jCb_language.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        jCb_language.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCb_languageActionPerformed(evt);
            }
        });
        jPanel2.add(jCb_language);
        jCb_language.setBounds(90, 10, 70, 20);

        jCB_accept.setText("Accept all licenses");
        jCB_accept.setName("jCB_accept"); // NOI18N
        jPanel2.add(jCB_accept);
        jCB_accept.setBounds(210, 10, 200, 23);

        jB_startApp.setText("Accept all licenses and start");
        jB_startApp.setName("jB_startApp"); // NOI18N
        jB_startApp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_startAppActionPerformed(evt);
            }
        });
        jPanel2.add(jB_startApp);
        jB_startApp.setBounds(110, 40, 280, 23);

        getContentPane().add(jPanel2);
        jPanel2.setBounds(0, 550, 500, 80);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jCb_languageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCb_languageActionPerformed
        // TODO add your handling code here:

        try
        {
            getAppliConf().changeLanguage( (String) jCb_language.getSelectedItem() );
        }
        catch( Throwable th )
        {
            th.printStackTrace();
        }
    }//GEN-LAST:event_jCb_languageActionPerformed

    private void jB_startAppActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_startAppActionPerformed
        // TODO add your handling code here:

        if( !jCB_accept.isSelected() )
        {
			this.
            jCB_accept.requestFocusInWindow();
			SwingUtilities.invokeLater( new Runnable(){ 
					@Override
					public void run() {
						highlightFocus();
				}
			});
        }
        else
        {
            _acceptedLicensesAndStart = true;
            getAppliConf().setLicencesHaveBeenAccepted(true);
			setVisible(false);
        }
    }//GEN-LAST:event_jB_startAppActionPerformed

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(GenericLicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(GenericLicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(GenericLicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(GenericLicenseJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the dialog */
/*		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				GenericLicenseJDialog dialog = new GenericLicenseJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
			}
		});
*/
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_startApp;
    private javax.swing.JCheckBox jCB_accept;
    private javax.swing.JComboBox jCb_language;
    private javax.swing.JLabel jL_language;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    // End of variables declaration//GEN-END:variables

	public abstract String getResourcePathToRtfLicense( String language );
	public abstract String getLongFileNameToRtfLicense( String language );
	public abstract String getBaseConfFileName( );

	public abstract String getRtfFileName();

	public void setRtfDocument()
	{
		String language = (String) jCb_language.getSelectedItem();
		setRtfDocument( language );
	}

	public void setRtfDocument( String language )
	{
		String longResourceName = getResourcePathToRtfLicense( language );
		String longFileName = getLongFileNameToRtfLicense( language );

		Document doc = null;

		doc = FileFunctions.loadRtfDocumentFromFile( longFileName );

		if( doc == null )
		{
			doc = ResourceFunctions.loadResourceRTFDocument( longResourceName );

			if( doc == null )
			{
				longResourceName = getResourcePathToRtfLicense( getAppliConf().getDefaultLanguage() );
				doc = ResourceFunctions.loadResourceRTFDocument( longResourceName );
			}

			if( doc != null )
			{
				File dir = ( new File( longResourceName ) ).getParentFile();
				dir.mkdirs();
				// if the file was not prevously created, we copy now the resource to the configuracion path.
				ResourceFunctions.copyBinaryResourceToFile( longResourceName, longFileName );
			}
		}

		_rtfDocJPanel.setRtfDocument( doc );
	}

	@Override
	public void formWindowClosingEvent( )
	{
		// we do nothing, because we need to check after the closure if the licenses have been accepted and we have to start.
		setVisible(false);
	}

	@Override
	public void changeLanguage( String language ) throws ConfigurationException, InternException
	{
		super.changeLanguage( language );

		setRtfDocument( language );
	}

	public void closeAndReleaseWindow()
	{
		formWindowClosing( true );
	}
	
}
